from fastapi import FastAPI, Query
from binance.client import Client
from datetime import datetime
from typing import List, Dict, Any, Optional

# Setup Binance API key and secret
api_key = 'r5fSFUsf6vry2j4Q2sIYLvQmqu34p85nE1EwjpeCt6rSj7HNHhKnX5vjcM75VAvd'
api_secret = 'tiiALx8RBhB59Ty4avf6yuQbdMK3GmQNdoMRrrlcERCFx2Y19brzTdl0PgnikxjU'

# Initialize the Binance client
client = Client(api_key, api_secret)

# Initialize FastAPI
app = FastAPI()

# Function to fetch balance
def get_balance() -> List[str]:
    try:
        account_info = client.get_account()
        balances = account_info['balances']
        assets = [balance['asset'] for balance in balances if float(balance['free']) > 0]
        return assets
    except Exception as e:
        print(f"Error fetching balance: {e}")
        return []

# Function to get order history for a specific symbol with 'FILLED' status
def get_order_history(symbol: str) -> List[Dict[str, Any]]:
    try:
        orders = client.get_all_orders(symbol=symbol)
        filled_orders = [order for order in orders if order['status'] == 'FILLED']
        filled_orders.sort(key=lambda x: x['time'], reverse=True)
        return [
            {
                "orderId": order['orderId'],
                "symbol": order['symbol'],
                "price": order['price'],
                "quantity": order['origQty'],
                "status": order['status'],
                "type": order['type'],
                "side": order['side'],
                "time": datetime.utcfromtimestamp(order['time'] / 1000).isoformat(),
            }
            for order in filled_orders
        ]
    except Exception as e:
        print(f"Error fetching order history for {symbol}: {e}")
        return []

# API endpoint to fetch order history for assets in balance
@app.get('/')
async def root():
    return {"message": "Hello World"}
    
@app.get("/order-history", response_model=List[Dict[str, Any]])
def get_order_history_for_balances(
    start_date: Optional[str] = Query(None, description="Start date in YYYY-MM-DD format"),
    end_date: Optional[str] = Query(None, description="End date in YYYY-MM-DD format")
):
    try:
        # Convert dates to datetime objects
        start_datetime = datetime.fromisoformat(start_date) if start_date else None
        end_datetime = datetime.fromisoformat(end_date) if end_date else None
        
        assets = get_balance()
        if not assets:
            return {"message": "No assets found in balance."}
        
        all_order_histories = []
        for asset in assets:
            symbol = f"{asset}USDT"
            history = get_order_history(symbol)
            
            # Filter orders based on date range
            if start_datetime or end_datetime:
                filtered_history = [
                    order for order in history
                    if (not start_datetime or datetime.fromisoformat(order['time']) >= start_datetime) and
                       (not end_datetime or datetime.fromisoformat(order['time']) <= end_datetime)
                ]
                all_order_histories.extend(filtered_history)
            else:
                all_order_histories.extend(history)
        
        return all_order_histories
    except Exception as e:
        return {"error": str(e)}
